
from pyreportgen.base import Component
import pyreportgen.helpers as helpers 


class Box(Component):
    def __init__(self, children: list[Component]=[], color:str="Info"):
        self.children:list[Component] = children
        self.color:str = color
    
    def render(self) -> str:
        html = ""
        for i in self.children:
            html += i.render()
        return helpers.tagwrap(html, "div", f"Box Color{self.color}")

class Empty(Component):
    def __init__(self):
        super().__init__()
    def render(self) -> str:
        return helpers.tagwrap("", "div")

class HBox(Component):
    def __init__(self, children:list[Component]=[]):
        super().__init__()
        self.children:list[Component] = children
    
    def render(self) -> str:
        html = ""

        for i in self.children:
            html += i.render()

        
        return helpers.tagwrap(html, "div", "HBox")

    
class VBox(Component):
    def __init__(self, children:list[Component]=[]):
        super().__init__()
        self.children:list[Component] = children
    
    def render(self) -> str:
        html = ""

        for i in self.children:
            html += i.render()

        return helpers.tagwrap(html, "div", "VBox NoBreak")
    
class HCenterContent(Component):
    def __init__(self, child:Component):
        super().__init__()
        self.child:Component = child
    def render(self) -> str:
        return helpers.tagwrap(self.child.render(), "div", "HCenterContent")
    
class PageBreak(Component):
    def __init__(self):
        super().__init__()
    
    def render(self) -> str:
        return helpers.tagwrap("", "div", "PageBreak")
    
class List(Component):
    def __init__(self, elements:list[str], numbered:bool=False, start:int=1):
        super().__init__()
        self.elements:list[str] = elements
        self.numbered:bool = numbered
        self.start:int = start

    def render(self) -> str:
        html = ""
        element = ""
        if self.numbered:
            element = "ol"
        else:
            element = "ul"


        for el in self.elements:
            html += helpers.tagwrap(el, "li")
        
        return helpers.tagwrap(html, element, props=f"start='{self.start}'")


STYLE_NORMAL = """
@import url('https://fonts.googleapis.com/css2?family=Poppins:wght@300;400;700&display=swap');

* {
    font-family: 'Poppins', sans-serif;
    margin: 0;
}

.CenterText {
    text-align: center;
}

.Main {
    padding: 0.75rem;
}


p, h1, h2, h3, h4, h5, h6 {
    margin: 0.5rem;
}

.HBox {
    display: -webkit-box;
    display: flex;
    -webkit-box-pack: center;
}

.HBox > * {
    -webkit-box-flex: 1;
    -webkit-flex: 1;
    flex: 1;
    
}

.HCenterContent {
    display: flex;
    justify-content: center;
}

.Image {
    width: 100%;
}

.HBarPlot {
    width: 100%;
}

.Map {
    width: 100%;
}

table {
    border-collapse: collapse;
    width: 100%;
}
  
td, th {
    border: 1px solid #dddddd;
    text-align: left;
    padding: 8px;
}
  
tr:nth-child(even) {
   background-color: #dddddd;
}

.tfoot {
    font-weight: bold;
}

.Box {

}

.Box.ColorInfo {
    background-color: rgb(223, 223, 223);
    border: 3px solid gray;
    border-radius: 10px;
}

@media print {
    .NoBreak {
        page-break-inside: avoid;
        break-inside: avoid;
    }
    .PageBreak {
        page-break-after: always;
    }
}

@media not print {
    .Main {
      width: 210mm;
      background-color: white;
    }
    body {
        display: flex;
        justify-content: center;
        background-color: gray;
    }
}
"""


import re
import os
import pyreportgen.style as style
import pyreportgen.layout as layout
import pyreportgen.helpers as helpers
import pyreportgen.statistic as statistic
from pyreportgen.base import Component, _DATA_DIR
import uuid
from email import encoders
from email.mime.base import MIMEBase
from email.mime.image import MIMEImage
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
import smtplib, ssl
import os

if not (_DATA_DIR in os.listdir()):
    os.makedirs(_DATA_DIR, exist_ok=True)


_RE_COMBINE_WHITESPACE = re.compile(r"\s+")



class Report(Component):
    def __init__(self, children:list[Component]=[], style:str=style.STYLE_NORMAL):
        super().__init__()
        self.children = children
        self.style = style

    def render(self) -> str:
        html = ""
        for i in self.children:
            html += i.render()
        html = f"""
            <!DOCTYPE html>
            <html lang="en">
            <head>
                <meta charset="UTF-8">
                <meta name="viewport" content="width=device-width, initial-scale=1.0">
                <style>{self.style}</style>
                <title>Report</title>
            </head>
            <body>
                <main class="Main">
                    {html}
                </main>
            </body>
        """

        return _RE_COMBINE_WHITESPACE.sub(" ", html).strip()
    
    def pdf(self, path:str, html:str=""):
        import pdfkit 
        if html == "":
            html = self.render()

        with open(_DATA_DIR+'/out.html', 'w', encoding="UTF-8") as f:
            f.write(html)

        pdfkit.from_file(_DATA_DIR+'/out.html', path, options={"--enable-local-file-access":None, "--print-media-type":None})
    
    def email(self, user:str, password:str, subject:str, sender:str, recipient:str, smtp_server:str, smtp_port:int, context=ssl.create_default_context(), include_pdf=False, pdfname:str="Report"):
        message = MIMEMultipart("alternative")
        message["Subject"] = subject
        message["From"] = sender
        message["To"] = recipient


        html = self.render()

        filenames = []

        image_lookup = {}

        # Get image tags
        for img in re.findall(r"<img [^>]*>", html):
            # Get the content of the src attribute.
            path = re.findall(r"(?<=src=')[^']*(?=')",img)[0]
            image_lookup[path] = str(uuid.uuid4()) 
            html = html.replace(path, f"cid:{image_lookup[path]}")
            filenames.append(path)



        # Turn these into plain/html MIMEText objects
        html_message = MIMEText(html, "html")

        # Add HTML/plain-text parts to MIMEMultipart message
        # The email client will try to render the last part first
        message.attach(html_message)

        for i in filenames:
            # This example assumes the image is in the current directory
            prefix = ""
            if not ((i.startswith("/")) or (i.startswith("*:\\"))):
                prefix = f"{_DATA_DIR}/"
            fp = open(f"{prefix}{i}", 'rb')
            msgImage = MIMEImage(fp.read())
            fp.close()

            # Define the image's ID as referenced above
            msgImage.add_header('Content-ID', f'<{image_lookup[i]}>')
            message.attach(msgImage)

        
        if include_pdf:

            # Open PDF file in binary mode
            self.pdf(f"{_DATA_DIR}/tmp.pdf")
            with open(f"{_DATA_DIR}/tmp.pdf", "rb") as attachment:
                # Add file as application/octet-stream
                # Email client can usually download this automatically as attachment
                pdf = MIMEBase("application", "octet-stream")
                pdf.set_payload(attachment.read())

            # Encode file in ASCII characters to send by email    
            encoders.encode_base64(pdf)

            # Add header as key/value pair to attachment part
            pdf.add_header(
                "Content-Disposition",
                f"attachment; filename={pdfname}.pdf",
            )



            # Add attachment to message and convert message to string
            message.attach(pdf)

        with smtplib.SMTP_SSL(smtp_server, smtp_port, context=context) as server:
            server.login(user, password)
            server.sendmail(sender, recipient, message.as_string())
    
class Html(Component):
    def __init__(self, html:str):
        super().__init__()
        self.html = html
    def render(self) -> str:
        return self.html
    
class ConditionalHtml(Component):
    def __init__(self, filepath:str, children:list[Component] = []):
        super().__init__()
        self.filepath:str = filepath
        self.children:list[Component] = children
    
    def render(self) -> str:
        if os.path.isfile(self.filepath):
            html = open(self.filepath, "r", encoding="UTF-8").read()
        else:
            html = ""
            for i in self.children:
                html += i.render()
        return helpers.tagwrap(html, "div")


class Text(Component):
    element = "p"

    def __init__(self, text:str, center=False):
        super().__init__()
        self.text = text
        self.center = center
    
    def render(self) -> str:
        classlist = ""
        if self.center:
            classlist += "CenterText "

        return helpers.tagwrap(self.text, self.element, classList=classlist)

class Header(Text):
    def __init__(self, text: str, center=True, heading=1):
        super().__init__(text, center)
        self.element = "h"+str(helpers.clamp(heading, 1, 6))
    
    def render(self) -> str:
        return super().render()

class Image(Component):
    def __init__(self, src: str):
        super().__init__()
        self.src: str = src
    
    def render(self) -> str:
        localizer = "../"
        if self.src.startswith("/"):
            localizer = ""
        return helpers.tagwrap("", "img", "Image", f"src='{localizer}{self.src}'", close=False)




_DATA_DIR = ".pyreportgen_data"


class Component:
    def __init__(self):
        pass

    def render(self) -> str:
        print(f"Component {self} has no renderer.")
        return f"""<p style='color: red'>Component {self.__repr__().replace("<", "&lt;").replace(">", "&gt;")} has no renderer</p>"""


from setuptools import setup, find_packages

import os
readme = open("./readme.md").read()

print(os.listdir())
print(find_packages("./pyreportgen/", include="__init__.py"))

setup(
    name="pyreportgen",
    version="0.1.6",
    description="A package for creating reports and other pdf documents.",
    long_description=readme,
    long_description_content_type='text/markdown',
    packages=find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.6",
)


from pyreportgen.base import Component, _DATA_DIR
import matplotlib.pyplot as plt
import numpy as np
import pyreportgen.helpers as helpers
import uuid

class HBarPlot(Component):
    def __init__(self, lables, data, title="", xlabel=""):
        super().__init__()
        self.lables = lables
        self.data = data
        self.title = title
        self.xlabel = xlabel
        self.path = helpers.random_path("png")

    def render(self) -> str:
        fig, ax = plt.subplots()

        y_pos = np.arange(len(self.lables))


        ax.barh(y_pos, self.data, align='center')
        ax.set_yticks(y_pos, labels=self.lables)
        ax.invert_yaxis() 
        ax.set_xlabel(self.xlabel)
        ax.set_title(self.title)

        plt.savefig(self.path, dpi=150)

        return helpers.tagwrap("", "img", "HBarPlot", f'src="{self.path.lstrip(_DATA_DIR+"/")}"', close=False)
    
class Table(Component):
    def __init__(self, data: list[list[str]], headers: list[str]=[], footers: list[str]=[]):
        self.data: list[list[str]] = data
        self.headers: list[str] = headers
        self.footers: list[str] = footers
    
    def render(self) -> str:
        tablecontent = ""
        
        headcontent = helpers.tagwrap("".join(
            [helpers.tagwrap(str(i), "th") for i in self.headers]
        ), "tr")
        
        for row in self.data:
            tablecontent += helpers.tagwrap("".join(
                [helpers.tagwrap(str(i), "td") for i in row]
        ), "tr")

        footcontent = helpers.tagwrap("".join(
            [helpers.tagwrap(str(i), "td") for i in self.footers]
        ), "tr", "tfoot")
        if len(self.footers) == 0:
            footcontent = ""
        if len(self.headers) == 0:
            headcontent = ""

        return helpers.tagwrap(headcontent+tablecontent+footcontent, "table", "Table NoBreak")